/*****************************************************************
* DS3231 RTC óra kezelés                                         *
* Csak három függvénye van, képes beállítani az aktuális dátumot *
* és időt, kiolvasni az aktuális dátumot és időt, valamint       *
* a hőmérsékletet                                                *
*****************************************************************/

#ifndef DS3231_CH55X_h
#define DS3231_CH55X_h

#include "Arduino.h"

byte bcd_Dec(byte);
byte dec_bcd(byte);
bool setTime24(unsigned int,byte,byte,byte,byte,byte,byte);         //rosszul megadott dátum vagy idő esetén 0-val tér vissza
                                                                    //Hónapot nem ellenőrzi pontosan, csak azt figyeli, hogy ne 
                                                                    //legyen negatív vagy 31-nél nagyobb.
bool getTime24(unsigned int*,byte*,byte*,byte*,byte*,byte*,byte*);  //A függvény a változók memória címét veszi át, 
                                                                    //és közvetlenül a változók tartalmát fogja módosítani.
                                                                    //0-át ad vissza, ha elemes táplálásnál leállta az oszcillátor
                                                                    //azaz az óra valószínűleg nem pontos. 1-et ad vissza, ha elemes
                                                                    //táplálásnál nem volt leállás, azaz az óra pontosan járt tovább.
float getTemperature();



byte bcd_Dec(byte val) {  // Convertál bcd számból decimálisba
  return ( (val/16*10) + (val%16) );
}
byte dec_bcd(byte val) {  // Convertál decimális számból bcd-be
  return ( (val/10*16) + (val%10) );
}

bool setTime24(unsigned int _ev, byte _ho, byte _nap, byte _het_napja, byte _ora, byte _perc, byte _masodperc) {
  //Ezzel a függvénnyel állítjuk be az időt, és a DS3231 alap állapotát is (24 órás üzemmód, négyszögjel kimenet stb.).
  //A dátumot és időt pontosan át kell adni, egyben állít be mindent. Az értékeküket allenőrzi, de a hónap napjainak
  //számát nem ellenőrzi pontosan. 
  //a változók tartalmának ellenőrzése. 
  if (_ev>2100 | _ho>12 | _nap>31 | _het_napja>7 | _ora>23 | _perc>59 | _masodperc>59) {
    return 0;
  }
  //status regiszter beállítása
  I2CStart();
  I2CSend(0x68 << 1 | 0);         //Írni fogunk a DS3231-re.
  I2CSend(0x0E);                  //A control regiszterbe fogunk írni, ez annak a címe a chip-ben
  I2CSend(0x63);                  //Ez a control regiszter új tartalma (ez az érték binárisan 0b01100011)
                                  //BBSQW bit 1, vagyis elemes táplálásnál is működik az oszcillátor
                                  //ha 1-re állítanánk, elemes táplálásnál nem járna az óra, csak őrizni az utolsó időpontot
                                  //A RS2 és RS1 bit 0-val beállítja a négyszögjel kimenetet 1Hz-ra, ezen lehet változtatni:
                                  // 0b01101011 esetén 1024Khz, 0b01110011 esetén 4096Khz, 0b01111011 esetén 8192Khz.
                                  //A INTCN bit 0-val beállítja, hogy az INT/SQW kimenet beállított frekvenciájú jelet ad ki.
  //címben a státus regiszter következik, nem kell megcímezni csak írni, és az adat oda kerül
  I2CSend(0x00);                  //A status regiszterben töröljük az OSF fleg-et, ami 1-el jelzi majd, ha elemes táplálásnál 
                                  //nem működött az oszcillátor, vagyis az óra valószínűleg nem pontos.
                                  //Az EN32Khz bit-et is 0-ra írjuk, azaz letiltjuk a 32Khz-s kimenetet
  I2CRestart();
  I2CSend(0x68 << 1 | 0);         //Írni fogunk a DS3231-re.
  I2CSend(0x00);                  //Most az adatregiszterekbe írjü1k a pontos időt, ez a legelső adatregiszter címe a chip-ben
  I2CSend(dec_bcd(_masodperc));
  I2CSend(dec_bcd(_perc));
  I2CSend(dec_bcd(_ora) & 0b10111111);
  I2CSend(dec_bcd(_het_napja)); 
  I2CSend(dec_bcd(_nap));
  I2CSend(dec_bcd(_ho));
  I2CSend(dec_bcd(_ev-2000)); 
  I2CStop();
  return 1;
}

bool getTime24(unsigned int* p_ev, byte* p_ho, byte* p_nap, byte* p_het_napja, byte* p_ora, byte* p_perc, byte* p_masodperc) {
  //Ezzel a függvénnyel visszakjuk az aktuális időt és dátumot. 24 órás formátumban és az évet évszázaddal kapjuk vissza.
  //A paramétereket referencia szerint adjuk át, vagyis a változó memória címeit. Ezért a függvény közvetlenül 
  //a változók tartalmát módosítja. 
  //kiolvassuk a dátumot és az időpontot 
  I2CStart();
  I2CSend(0x68 << 1 | 0);     //Írni fogunk a DS3231-re.
  I2CSend(0x00);              //az adatregisztereket fogjuk kiolvasni, ez a kezdőcím a chip-ben
  I2CRestart();
  I2CSend(0x68 << 1 | 1);     //Olvasni fogunk a DS3231-ből.
  *p_masodperc = bcd_Dec(I2CRead());  //A mutató átlat tárolt memóriacímen található byte típusú változó tartalmát módosítjuk
  I2CAck();
  *p_perc = bcd_Dec(I2CRead());
  I2CAck();
  *p_ora = bcd_Dec(I2CRead());
  I2CAck();
  *p_het_napja = bcd_Dec(I2CRead());
  I2CAck();
  *p_nap = bcd_Dec(I2CRead());
  I2CAck();
  *p_ho = bcd_Dec(I2CRead() & 0b01111111);
  I2CAck();
  *p_ev = bcd_Dec(I2CRead())+2000;   //A mutató átlat tárolt memóriacímen található int típusú változó tartalmát módosítjuk
  I2CNak();
  I2CRestart();
  //Kiolvassuk a status regiszterből a osf flag-et, ami megmondja, ha elemes táplálásnál leállt az oszcillátor
  I2CSend(0x68 << 1 | 0);     //Írni fogunk a DS3231-re.
  I2CSend(0x0F);              //ez a státus regiszter címe
  I2CRestart();
  I2CSend(0x68 << 1 | 1);     //Olvasni fogunk a DS3231-ből.
  if (I2CRead() & 0b10000000) {return 0;} else {return 1;}  //Kiolvassuk a status regisztert, kimaszkoljuk az OSF bitet.
                                                            //Ez alapján 1-el jelezzük, ha az óra valószínűleg pontos
  I2CStop();
}

float getTemperature() {
  // Kiolvassa a hőmérséklet értékét a 0x11h és 0x12h regiszterekből
  float temp;
  I2CStart();
  I2CSend(0x68 << 1 | 0);                      //Írni fogunk a DS3231-re.
  I2CSend(0x11);                               //a DS3231 chip 0x11 és 0x12-es regisztere a hőmérséklet regiszter
  I2CRestart();
  I2CSend(0x68 << 1 | 1);                      //Olvasni fogunk a DS3231-ből.
  temp=I2CRead()+(0.25*(float)(I2CRead()>>6)); //hőmérséklet MSB és LSB regisztere kiolvasás, hőmérséklet számítás
  I2CStop();
  return temp;                                 //visszatérési érték
}

#endif
